/*global __DEV__*/
const __DEV__ = true;
'use strict';

const isDev = typeof __DEV__ === 'boolean' && __DEV__;
import TextStylePropTypes from './validation/TextStylePropTypes';
import ImageStylePropTypes from './validation/ImageStylePropTypes';
import ViewStylePropTypes from './validation/ViewStylePropTypes';
import WeexProps from './validation/WeexProps';
import PropTypeLocationNames from './validation/PropTypeLocationNames';

import chalk from 'chalk';


let PropTypesSecret = 'SECRET_DO_NOT_PASS_THIS_OR_YOU_WILL_BE_FIRED';

class StyleSheetValidation {
  static validateStyleProp(prop, style, caller) {
    if (!isDev || typeof style[prop] === 'object') {
      return;
    }

    // weex 不支持的属性
    if (WeexProps.noSupport.indexOf(prop) > -1) {
      let message1 = '"' + prop + '" 在weex环境下不可用.';
      styleError(message1, style, caller, '', 'color: red; font-size: 15px');
      console.log(chalk.yellow('查看文档：http://alibaba.github.io/weex/doc/references/common-style.html'));
    }
    // 只有 weex 有的属性
    if (WeexProps.particular.indexOf(prop) > -1) {
      let message1 = '"' + prop + '" 只在weex环境下可用.';
      styleError(message1, style, caller);
      return;
    }
    if (allStylePropTypes[prop] === undefined) {
      let message1 = '"' + prop + '" 不是一个可用的样式属性.';
      let message2 = '\nValid style props: ' +
        JSON.stringify(Object.keys(allStylePropTypes).sort(), null, '  ');
      styleError(message1, style, caller, message2);
      return;
    }
    let error = allStylePropTypes[prop](
      style,
      prop,
      caller,
      PropTypeLocationNames.prop,
      null,
      PropTypesSecret
    );
    if (error) {
      styleError(error.message, style, caller);
    }
  }

  static validateStyle(name, styles) {
    if (!isDev) {
      return;
    }
    for (let prop in styles[name]) {
      StyleSheetValidation.validateStyleProp(prop, styles[name], 'StyleSheet ' + name);
    }
  }

  static addValidStylePropTypes(stylePropTypes) {
    for (let key in stylePropTypes) {
      allStylePropTypes[key] = stylePropTypes[key];
    }
  }
}

let styleError = function(message1, style, caller, message2, css) {
  if (css) {
    console.log(chalk.red(message1));
  } else {
    console.log(chalk.red(message1));
  }
};

let allStylePropTypes = {};

StyleSheetValidation.addValidStylePropTypes(ImageStylePropTypes);
StyleSheetValidation.addValidStylePropTypes(TextStylePropTypes);
StyleSheetValidation.addValidStylePropTypes(ViewStylePropTypes);

module.exports = StyleSheetValidation;