'use strict';

import PropTypeLocationNames from './PropTypeLocationNames';
import normalizeColor from '../normalizeColor';

function colorPropType(isRequired, props, propName, componentName, location, propFullName) {
  let color = props[propName];
  if (color === undefined || color === null) {
    if (isRequired) {
      let locationName = PropTypeLocationNames[location];
      return new Error(
        'Required ' + locationName + ' `' + (propFullName || propName) +
        '` was not specified in `' + componentName + '`.'
      );
    }
    return;
  }

  if (typeof color === 'number') {
    return;
  }

  if (normalizeColor(color) === null) {
    let locationName = PropTypeLocationNames[location];
    return new Error(
      'Invalid ' + locationName + ' `' + (propFullName || propName) +
      '` supplied to `' + componentName + '`: ' + color + '\n' +
`Valid color formats are
  - '#f0f' (#rgb)
  - '#f0fc' (#rgba)
  - '#ff00ff' (#rrggbb)
  - '#ff00ff00' (#rrggbbaa)
  - 'rgb(255, 255, 255)'
  - 'rgba(255, 255, 255, 1.0)'
  - 'hsl(360, 100%, 100%)'
  - 'hsla(360, 100%, 100%, 1.0)'
  - 'transparent'
  - 'red'
  - 0xff00ff00 (0xrrggbbaa)
`);
  }
};

let ColorPropType = colorPropType.bind(null, false /* isRequired */);
ColorPropType.isRequired = colorPropType.bind(null, true /* isRequired */);

export default ColorPropType;
